#ifndef _MBS
#define _MBS
#include <Arduino.h>
#include <stdint.h>
#include <SoftwareSerial.h>

//MODBUS function codes
#define READ_DO 	0x01
#define READ_DI		0x02
#define READ_AO 	0x03
#define READ_AI		0x04

#define WRITE_DO	0x05
#define WRITE_AO	0x06

#define MB_BUFLEN 200 //TODO: Trim to minimum necessary to save memory

class ModbusServer: public SoftwareSerial
{
public:
	ModbusServer(byte id, byte rxpin, byte txpin, uint32_t baud,
			uint16_t nbrHregs, uint16_t nbrCoils, int * hregs, byte * coils);
	void run(void);
	inline uint32_t getErrorCount(void)
	{
		return _errcnt;
	}
	inline void clrErrorCount(void)
	{
		_errcnt = 0;
	}

private:
	void calcCrc(void);
	void checkSerial(void);
	void serialRx(void);
	void getDigitalStatus(byte, uint16_t, uint16_t);
	void getAnalogStatus(byte, uint16_t, uint16_t);
	void setStatus(byte, uint16_t, uint16_t);
	//SoftwareSerial * mbSerial;   //Serial driver
	byte _device;       //modBus device number
	uint32_t _baud;      //Serial link speed

	byte * _coils;	    //modBus boolean table pointer
	int * _hregs;       //modBus analog table pointer
	uint16_t _nbrCoils; //modBus boolean table length
	uint16_t _nbrHregs; //modBus boolean table length

	byte _msg[MB_BUFLEN];   //modbus buffer
	byte _len;              //nbr of bytes in modbus buffer

	uint16_t _crc;          //CRC value
	uint16_t _frameDelay;   //Delay between chars
	uint32_t _errcnt;       //Crc error counter
};
#endif
